@@ -222,7 +222,11 @@ iso_message(existence_error(Type, Object, In)) --> % not ISO
 iso_message(busy(Type, Object)) -->
     [ '~w `~p'' is busy'-[Type, Object] ].
 iso_message(syntax_error(swi_backslash_newline)) -->
-    [ 'Deprecated ... \\<newline><white>*.  Use \\c' ].
+    [ 'Deprecated: ... \\<newline><white>*.  Use \\c' ].
+iso_message(syntax_error(warning_var_tag)) -->
+    [ 'Deprecated: dict with unbound tag (_{...}).  Mapped to #{...}.' ].
+iso_message(syntax_error(var_tag)) -->
+    [ 'Syntax error: dict syntax with unbound tag (_{...}).' ].
 iso_message(syntax_error(Id)) -->
     [ 'Syntax error: ' ],
     syntax_error(Id).
@@ -636,7 +640,9 @@ swi_comment(Msg) -->
 
 
 thread_context -->
-    { thread_self(Me), Me \== main, thread_property(Me, id(Id)) },
+    { \+ current_prolog_flag(toplevel_thread, true),
+      thread_self(Id)
+    },
     !,
     ['[Thread ~w] '-[Id]].
 thread_context -->
@@ -663,6 +669,9 @@ unwind_message(Term) -->
                  *        NORMAL MESSAGES       *
                  *******************************/
 
+:- dynamic prolog:version_msg/1.
+:- multifile prolog:version_msg/1.
+
 prolog_message(welcome) -->
     [ 'Welcome to SWI-Prolog (' ],
     prolog_message(threads),
@@ -1344,15 +1353,27 @@ query_result(yes(Bindings, Delays, Residuals)) -->
 query_result(more(Bindings, Delays, Residuals)) -->
     result(Bindings, Delays, Residuals),
     prompt(more, Bindings, Delays, Residuals).
+:- if(current_prolog_flag(emscripten, true)).
 query_result(help) -->
     [ ansi(bold, '  Possible actions:', []), nl,
-      '  ; (n,r,space,TAB): redo              | t:         trace&redo'-[], nl,
-      '  *:                 show choicepoint  | c (a,RET): stop'-[], nl,
-      '  w:                 write             | p:         print'-[], nl,
-      '  +:                 max_depth*10      | -:         max_depth//10'-[], nl,
-      '  b:                 break             | h (?):     help'-[],
+      '  ; (n,r,space): redo              | t:       trace&redo'-[], nl,
+      '  *:             show choicepoint  | . (c,a): stop'-[], nl,
+      '  w:             write             | p:       print'-[], nl,
+      '  +:             max_depth*5       | -:       max_depth//5'-[], nl,
+      '  h (?):         help'-[],
       nl, nl
     ].
+:- else.
+query_result(help) -->
+    [ ansi(bold, '  Possible actions:', []), nl,
+      '  ; (n,r,space,TAB): redo              | t:           trace&redo'-[], nl,
+      '  *:                 show choicepoint  | . (c,a,RET): stop'-[], nl,
+      '  w:                 write             | p:           print'-[], nl,
+      '  +:                 max_depth*5       | -:           max_depth//5'-[], nl,
+      '  b:                 break             | h (?):       help'-[],
+      nl, nl
+    ].
+:- endif.
 query_result(action) -->
     [ 'Action? '-[], flush ].
 query_result(confirm) -->
@@ -1520,7 +1541,7 @@ extra_line -->
 
 prolog_message(if_tty(Message)) -->
     (   {current_prolog_flag(tty_control, true)}
-    ->  [ at_same_line | Message ]
+    ->  [ at_same_line ], list(Message)
     ;   []
     ).
 prolog_message(halt(Reason)) -->
@@ -1585,13 +1606,14 @@ user_version_message(Atom) -->
                  *******************************/
 
 prolog_message(spy(Head)) -->
-    { goal_to_predicate_indicator(Head, Pred)
-    },
-    [ 'Spy point on ~p'-[Pred] ].
+    [ 'New spy point on ' ],
+    goal_predicate(Head).
+prolog_message(already_spying(Head)) -->
+    [ 'Already spying ' ],
+    goal_predicate(Head).
 prolog_message(nospy(Head)) -->
-    { goal_to_predicate_indicator(Head, Pred)
-    },
-    [ 'Spy point removed from ~p'-[Pred] ].
+    [ 'Removed spy point from ' ],
+    goal_predicate(Head).
 prolog_message(trace_mode(OnOff)) -->
     [ 'Trace mode switched to ~w'-[OnOff] ].
 prolog_message(debug_mode(OnOff)) -->
@@ -1864,6 +1886,8 @@ deprecated(source_search_working_directory(File, _FullFile)) -->
       'the Prolog flag ',
       ansi(code, source_search_working_directory, []), '.', nl
     ].
+deprecated(moved_library(Old, New)) -->
+    [ 'Library was moved: ~q --> ~q'-[Old, New] ].
 
 load_file(File) -->
     { file_base_name(File, Base),
@@ -1979,6 +2003,9 @@ code(Term) -->
 code(Format, Term) -->
     [ ansi(code, Format, [Term]) ].
 
+list([]) --> [].
+list([H|T]) --> [H], list(T).
+
 
 		 /*******************************
 		 *        DEFAULT THEME		*
@@ -2213,8 +2240,8 @@ add_message_context1(time(Format), Prefix0, Prefix) :-
     format_time(string(S), Format, Now),
     atomics_to_string([Prefix0, S, ' '], Prefix).
 add_message_context1(thread, Prefix0, Prefix) :-
+    \+ current_prolog_flag(toplevel_thread, true),
     thread_self(Id0),
-    Id0 \== main,
     !,
     (   atom(Id0)
     ->  Id = Id0
